//
//  ViewController.swift
//  Test
//
//  Created by Pedro Carrasco on 13/11/2019.
//  Copyright © 2019 Pedro Carrasco. All rights reserved.
//

import UIKit
import Carbon

class CarbonViewController: UIViewController {
    
    @IBOutlet var tableView: UITableView!
    let renderer = Renderer(adapter: UITableViewAdapter(), updater: UITableViewReloadDataUpdater())
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        title = "Carbon"
        
        tableView.sectionHeaderHeight = UITableView.automaticDimension
        tableView.estimatedSectionHeaderHeight = UITableView.automaticDimension

        renderer.target = tableView
        renderer.render {
            Section(id: 0, header: HeaderComponent()) {
                CellComponent()
            }
            
            Section(id: 1, header: HeaderComponent()) {
                CellComponent()
            }
        }
    }
}

struct CellComponent: IdentifiableComponent, Hashable {
    
    func renderContent() -> Cell {
        Cell()
    }
    
    func render(in content: Cell) {}
}


// MARK: - TextHeaderView
final class Cell: UIView {
    
    // MARK: Outlets
    private let label = UILabel()
    
    // MARK: - Lifecycle
    init() {
        super.init(frame: .zero)
        backgroundColor = UIColor.red.withAlphaComponent(0.25)
        addSubview(label)
        label.translatesAutoresizingMaskIntoConstraints = false
        label.numberOfLines = 0
        label.text = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Etiam eget ligula eu lectus lobortis condimentum. Aliquam nonummy auctor massa. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nulla at risus. Quisque purus magna, auctor et, sagittis ac, posuere eu, lectus. Nam mattis, felis ut adipiscing"
        NSLayoutConstraint.activate(
            [
                label.topAnchor.constraint(equalTo: self.topAnchor),
                label.bottomAnchor.constraint(equalTo: self.bottomAnchor),
                label.leadingAnchor.constraint(equalTo: self.leadingAnchor),
                label.trailingAnchor.constraint(equalTo: self.trailingAnchor)
            ]
        )
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}

struct HeaderComponent: IdentifiableComponent, Hashable {
    
    init() {}
    
    func renderContent() -> Header {
        Header()
    }
    
    func render(in content: Header) {}
}


// MARK: - TextHeaderView
final class Header: UIView {
    
    // MARK: Outlets
    private let label = UILabel()
    
    // MARK: - Lifecycle
    init() {
        super.init(frame: .zero)
        backgroundColor = UIColor.blue.withAlphaComponent(0.25)
        addSubview(label)
        label.translatesAutoresizingMaskIntoConstraints = false
        label.numberOfLines = 0
        label.text = "This is an header"
        NSLayoutConstraint.activate(
            [
                label.topAnchor.constraint(equalTo: topAnchor, constant: 16.0),
                label.leadingAnchor.constraint(equalTo: leadingAnchor),
                label.trailingAnchor.constraint(equalTo: trailingAnchor),
                label.bottomAnchor.constraint(equalTo: bottomAnchor)
            ]
        )
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}
